local DialogTweaks = ZO_Object:Subclass()

DialogTweaks.defaults = {
   ["numberedOptions"] = true,
   ["notSoVisibleGoodbye"] = true,
   ["removeDashes"] = true,
   ["nameAlignment"] = "center",
   ["nameColor"] = { 
      ["r"] = ZO_HIGHLIGHT_TEXT.r,
      ["g"] = ZO_HIGHLIGHT_TEXT.g,
      ["b"] = ZO_HIGHLIGHT_TEXT.b,
      ["a"] = ZO_HIGHLIGHT_TEXT.a,
   }
}
DialogTweaks.alignments = {
   "left",
   "center",
   "right"
}
DialogTweaks.options = {
   [1] = "ZO_ChatterOption1",
   [2] = "ZO_ChatterOption2",
   [3] = "ZO_ChatterOption3",
   [4] = "ZO_ChatterOption4",
   [5] = "ZO_ChatterOption5",
   [6] = "ZO_ChatterOption6",
   [7] = "ZO_ChatterOption7",
   [8] = "ZO_ChatterOption8",
   [9] = "ZO_ChatterOption9",
   [10] = "ZO_ChatterOption10"
}

function DialogTweaks:Style()
   local lastOption = 0
   for k, v in pairs(self.options) do
      local option = _G[v]
      if self.config.numberedOptions then
         option:SetText(tostring(k .. " - " .. option:GetText()))
      end
      if option:IsHidden() == true and lastOption == 0 then
         lastOption = k - 1
      end
   end
   if self.config.notSoVisibleGoodbye then
      _G[self.options[lastOption]]:SetColor(ZO_DEFAULT_DISABLED_COLOR:UnpackRGBA())
   end

   local npc = ZO_InteractWindowTargetAreaTitle
   npc:SetColor(self.config.nameColor["r"], self.config.nameColor["g"], self.config.nameColor["b"], self.config.nameColor["a"])
  
   if self.config.removeDashes then
      local npcName = npc:GetText()
      local prettyName = string.sub(npcName, 2, string.len(npcName) - 1)
      npc:SetText(prettyName)
   end

   if self.config.nameAlignment == "left" then
      npc:SetHorizontalAlignment(TEXT_ALIGN_LEFT)
   elseif self.config.nameAlignment == "center" then
      npc:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
   elseif self.config.nameAlignment == "right" then
      npc:SetHorizontalAlignment(TEXT_ALIGN_RIGHT)
   end
end

function DialogTweaks:Init(eventCode, addonName)
   if addonName == "DialogTweaks" then
      self.config = ZO_SavedVars:New("DialogTweaksSavedVars", 1, nil, self.defaults, nil)

      ZO_InteractWindowPlayerAreaOptions:RegisterForEvent(EVENT_CHATTER_BEGIN, function() self:Style() end)
      ZO_InteractWindowPlayerAreaOptions:RegisterForEvent(EVENT_CONVERSATION_UPDATED, function() self:Style() end)
      ZO_InteractWindowPlayerAreaOptions:RegisterForEvent(EVENT_QUEST_COMPLETE_DIALOG, function() self:Style() end)
      ZO_InteractWindowPlayerAreaOptions:RegisterForEvent(EVENT_QUEST_OFFERED, function() self:Style() end)

      self:CreateSettings()
   end
end

function DialogTweaks:CreateSettings()
   local panelData = {
      type = 'panel',
      name = "Biki's Dialog Tweaks",
      displayName = ZO_HIGHLIGHT_TEXT:Colorize("Biki's Dialog Tweaks"),
      author = 'Biki',
      version = '1.3b',
      slashCommand = "/dialogtweaks",
      registerForRefresh = true,
      registerForDefaults = true,
   }
   local optionsData = {
      {
         type = 'checkbox',
         name = 'Number the options',
         tooltip = 'Numbers the options in a dialog beginning with 1. You can press the corresponding key to select that option',
         getFunc = function() return self.config.numberedOptions end,
         setFunc = function(value) self.config.numberedOptions = value end,
         default = self.defaults.numberedOptions,
      },
      {
         type = 'checkbox',
         name = 'Gray out "Goodbye"',
         tooltip = 'Grays out the Goodbye option a bit so it is less distracting',
         getFunc = function() return self.config.notSoVisibleGoodbye end,
         setFunc = function(value) self.config.notSoVisibleGoodbye = value end,
         default = self.defaults.notSoVisibleGoodbye,
      },
      {
         type = 'checkbox',
         name = 'Remove dashes from NPC name',
         tooltip = 'Removes the dashes (-) from the NPC/target name',
         getFunc = function() return self.config.removeDashes end,
         setFunc = function(value) self.config.removeDashes = value end,
         default = self.defaults.removeDashes,
      },
      {
         type = 'dropdown',
         name = 'Alignment of NPC name',
         tooltip = 'Select the alignment of the NPC/target name',
         choices = self.alignments,
         getFunc = function() return self.config.nameAlignment end,
         setFunc = function(value) self.config.nameAlignment = value end,
         default = self.defaults.nameAlignment,
      },
      {
         type = 'colorpicker',
         name = 'NPC name/target color',
         tooltip = 'Adjust the color of the NPC/target name',
         getFunc = function() return self.config.nameColor["r"], self.config.nameColor["g"], self.config.nameColor["b"], self.config.nameColor["a"] end,
         setFunc = function(r, g, b, a) 
            self.config.nameColor["r"] = r
            self.config.nameColor["g"] = g
            self.config.nameColor["b"] = b
            self.config.nameColor["a"] = a
         end,
         default = self.defaults.nameColor,
      },
   }
   local LAM2 = LibStub("LibAddonMenu-2.0")
   LAM2:RegisterAddonPanel("_bikisDialogTweaks", panelData)
   LAM2:RegisterOptionControls("_bikisDialogTweaks", optionsData)
end

EVENT_MANAGER:RegisterForEvent("DialogTweaks", EVENT_ADD_ON_LOADED, function(...) DialogTweaks:Init(...) end)
